import 'dart:async';
import 'dart:io';
import 'dart:typed_data';
import 'package:http/http.dart' as ht;
import 'package:flutter/material.dart';
import 'package:flutter_email_sender/flutter_email_sender.dart';
import 'package:flutter_pdfview/flutter_pdfview.dart';
import 'package:path_provider/path_provider.dart';
import 'package:xcommonwidgets/xcommonwidgets.dart';
import 'package:xfxbase/XFDataItem.dart';
import 'package:xfxwidgets/XTabItem.dart';
import 'package:xtoolspackage/xtoolspackage.dart';

class XpdfScreenItem extends Widget {
  final String title;
  final Widget widgetToView;
  XpdfScreenItem({
    required this.title,
    required this.widgetToView,
  });

  @override
  Element createElement() {
    throw UnimplementedError();
  }
}

class XPDFScreen<T extends XFDataItem> extends StatefulWidget {
  final List<XpdfScreenItem>? liWidgetPDF;
  final String? title;
  final T? itemInEdit;
  final bool btnChangePDF_Visible;
  Email Function(T, List<File>)? email;
  List<Widget>? appBar_liActionWidgets;
  bool sendEmailBTN_visible;
  String urlPDF;

  XPDFScreen({
    Key? key,
    this.liWidgetPDF,
    this.itemInEdit,
    this.email,
    this.title,
    this.appBar_liActionWidgets,
    this.btnChangePDF_Visible = true,
    this.sendEmailBTN_visible = true,
    this.urlPDF = "",
  }) : super(key: key);

  _XPDFScreenState<T> createState() => _XPDFScreenState<T>();
}

class _XPDFScreenState<T extends XFDataItem> extends State<XPDFScreen<T>> with WidgetsBindingObserver, SingleTickerProviderStateMixin {
  final Completer<PDFViewController> _controller = Completer<PDFViewController>();
  int? pages = 0;
  int? currentPage = 0;
  bool isReady = false;
  String errorMessage = '';
  bool isBusy = false;

  ///Per gestire i Tab
  int currentTabIndex = 0;
  late TabController tabController;
  List<XTabItem> xTabs = List<XTabItem>.empty(growable: true);
  TickerProvider? vsync;
  Email? email;
  Widget? widgetToShow;
  List<File> liAllegati = List<File>.empty(growable: true);

  String PDFActive = "";
  @override
  void initState() {
    widgetToShow = widget.liWidgetPDF!.first.widgetToView;
    PDFActive = widget.liWidgetPDF!.first.title;
    super.initState();
  }

  Future<List<int>?> fetchByteDataFromUrl(String url) async {
    var response = await ht.get(Uri.parse(url));
    if (response.statusCode == 200) {
      return response.bodyBytes;
    } else {
      return null;
    }
  }

  _sendingMails() async {
    liAllegati.clear();
    List<int>? byteData = await fetchByteDataFromUrl(widget.urlPDF);
    if (byteData != null) {
      var pdfBytes = Uint8List.fromList(byteData);

      final directory = await getApplicationDocumentsDirectory();
      final path = directory.path;
      var fName;
      if (PDFActive.contains("Cliente")) {
        fName = '$path/Preventivo.pdf';
      } else {
        fName = '$path/Preventivo_Rivenditore.pdf';
      }
      var f = File(fName);
      var ff = await f.writeAsBytes(pdfBytes, flush: true, mode: FileMode.write);
      liAllegati.add(ff);
    }
    email = widget.email!(widget.itemInEdit!, liAllegati);
    await FlutterEmailSender.send(email!);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.grey[900],
      appBar: AppBar(
        actions: <Widget>[
          widget.appBar_liActionWidgets != null ? Row(mainAxisSize: MainAxisSize.min, mainAxisAlignment: MainAxisAlignment.end, children: widget.appBar_liActionWidgets!.map((e) => e).toList()) : Container(),
          widget.btnChangePDF_Visible
              ? XBtnbase(
                  margin: EdgeInsets.symmetric(vertical: 8, horizontal: 10),
                  decoration: BoxDecoration(border: Border.all(color: Colors.white)),
                  padding: EdgeInsets.all(3),
                  toolTip_Message: "Invia un Email con tutti e due i preventivi",
                  width: MediaQuery.of(context).size.width / 2,
                  label: "Vai al " + (PDFActive == widget.liWidgetPDF!.last.title ? "PDF del Cliente" : "PDF del Rivenditore"),
                  onPressed: isBusy.not()
                      ? () {
                          setState(() {
                            isBusy = true;
                          });
                          if (PDFActive == widget.liWidgetPDF!.first.title) {
                            PDFActive = widget.liWidgetPDF!.last.title;
                          } else if (PDFActive == widget.liWidgetPDF!.last.title) {
                            PDFActive = widget.liWidgetPDF!.first.title;
                          }
                          Timer.periodic(Duration(seconds: 1), (timer) {
                            setState(() {
                              isBusy = false;
                              widget.liWidgetPDF;
                            });
                            timer.cancel();
                          });
                        }
                      : null)
              : Container(),
          SizedBox(
            width: 20,
          ),
          widget.sendEmailBTN_visible
              ? IconButton(
                  icon: Icon(Icons.send_outlined),
                  tooltip: "Invia per Email il ${PDFActive}",
                  onPressed: () {
                    _sendingMails();
                  },
                )
              : Container(),
        ],
      ),
      body: isBusy
          ? XStatefulWidgetBase.LoaderIndicator(isBusy)
          : (PDFActive == widget.liWidgetPDF!.first.title
              ? widget.liWidgetPDF!.first.widgetToView
              : PDFActive == widget.liWidgetPDF!.last.title
                  ? widget.liWidgetPDF!.last.widgetToView
                  : Container()),
      floatingActionButton: (pages != null && pages! > 1)
          ? FutureBuilder<PDFViewController>(
              future: _controller.future,
              builder: (context, AsyncSnapshot<PDFViewController> snapshot) {
                if (snapshot.hasData) {
                  return FloatingActionButton.extended(
                    label: Text("Pagina ${pages! ~/ 2}"),
                    onPressed: () async {
                      await snapshot.data!.setPage(pages! ~/ 2);
                    },
                  );
                }
                return Container();
              },
            )
          : null,
    );
  }
}
